@@ -1,6 +1,6 @@
 from __future__ import print_function
 from .unit import Unit
-from .util import convert_to_ul, quad_ind_to_num
+from .util import quad_ind_to_num
 import sys
 
 if sys.version_info[0] >= 3:
@@ -87,9 +87,10 @@ class Well(object):
             Theoretical volume to indicate for a Well.
 
         """
-        v = convert_to_ul(vol)
-        if v > Unit(
-                self.container.container_type.well_volume_ul, "microliter"):
+        if not isinstance(vol, basestring) and not isinstance(vol, Unit):
+            raise TypeError("Volume given is not of type str or Unit. %s" % type(vol))
+        v = Unit.fromstring(vol)
+        if v > self.container.container_type.well_volume_ul:
             raise ValueError("Theoretical volume you are trying to set "
                              "exceeds the maximum volume of this well.")
         self.volume = v
@@ -147,6 +148,12 @@ class WellGroup(object):
             wells = [wells]
         elif isinstance(wells, WellGroup):
             wells = wells.wells
+        elif isinstance(wells, list):
+            if not all(isinstance(well, Well) for well in wells):
+                raise TypeError("All elements in list must be wells")
+        else:
+            raise TypeError("Wells must be Well, list of wells, WellGroup.")
+
         self.wells = wells
 
     def set_properties(self, properties):
@@ -265,7 +272,7 @@ class WellGroup(object):
             raise TypeError("You can only add a Well or WellGroups \
                                 together.")
         if isinstance(other, Well):
-            return WellGroup(self.wells.append(other))
+            return WellGroup(self.wells + [other])
         else:
             return WellGroup(self.wells + other.wells)
 
@@ -358,9 +365,9 @@ class Container(object):
         `ContainerType.robotize()`_ for more information.
 
         """
-        if not isinstance(well_ref, (basestring, int, Well)):
+        if not isinstance(well_ref, (basestring, int, Well, list)):
             raise TypeError("Well reference given is not of type 'str' \
-                            'int', or 'Well'.")
+                            'int', 'Well' or 'list'.")
         return self.container_type.robotize(well_ref)
 
     def humanize(self, well_ref):
@@ -372,8 +379,9 @@ class Container(object):
         `ContainerType.humanize()`_ for more information.
 
         """
-        if not isinstance(well_ref, int):
-            raise TypeError("Well reference given is not of type 'int'.")
+        if not isinstance(well_ref, (int, basestring, list)):
+            raise TypeError("Well reference given is not of type 'int',"
+                            "'str' or 'list'.")
         return self.container_type.humanize(well_ref)
 
     def decompose(self, well_ref):
@@ -426,15 +434,15 @@ class Container(object):
         num_rows = self.container_type.row_count()
         inner_wells = []
         if columnwise:
-            for c in xrange(1, num_cols-1):
+            for c in xrange(1, num_cols - 1):
                 wells = []
-                for r in xrange(1, num_rows-1):
-                    wells.append((r*num_cols)+c)
+                for r in xrange(1, num_rows - 1):
+                    wells.append((r * num_cols) + c)
                 inner_wells.extend(wells)
         else:
             well = num_cols
-            for i in xrange(1, num_rows-1):
-                inner_wells.extend(xrange(well+1, well+(num_cols-1)))
+            for i in xrange(1, num_rows - 1):
+                inner_wells.extend(xrange(well + 1, well + (num_cols - 1)))
                 well += num_cols
         inner_wells = [self._wells[x] for x in inner_wells]
         return WellGroup(inner_wells)
@@ -516,6 +524,59 @@ class Container(object):
 
         return self.wells(wells)
 
+    def set_storage(self, storage):
+        """
+        Set the storage condition of a container, will overwrite
+        an existing storage condition, will remove discard True.
+
+        Parameters
+        ----------
+        storage : str
+            Storage condition.
+
+        Raises
+        ----------
+        TypeError :
+            If storage condition not of type str.
+
+        """
+        if not isinstance(storage, basestring):
+            raise TypeError("Storage condition given ({0}) is not of type str. {1}."
+                            .format(storage, type(storage)))
+
+        self.storage = storage
+        return self
+
+    def discard(self):
+        """
+        Set the storage condition of a container to None and
+        container to be discarded if ref in protocol.
+
+        Example
+        ----------
+
+            .. code-block:: python
+
+                p = Protocol()
+                container = p.ref("new_container", cont_type="96-pcr", storage="cold_20")
+                p.incubate(c, "warm_37", "30:minute")
+                container.discard()
+
+                Autoprotocol generated:
+
+                .. code-block:: json
+
+                   "refs": {
+                      "new_container": {
+                        "new": "96-pcr",
+                        "discard": true
+                      }
+                    }
+
+        """
+        self.storage = None
+        return self
+
     def __repr__(self):
         """
         Return a string representation of a Container using the specified name.
