@@ -2,6 +2,7 @@ import re
 import sys
 from collections import namedtuple
 from .container import Well
+from .unit import Unit
 
 if sys.version_info[0] >= 3:
     xrange = range
@@ -19,7 +20,8 @@ class ContainerType(namedtuple("ContainerType",
                     ["name", "is_tube", "well_count",
                      "well_depth_mm", "well_volume_ul",
                      "well_coating", "sterile", "capabilities",
-                     "shortname", "col_count", "dead_volume_ul", "safe_min_volume_ul"])):
+                     "shortname", "col_count", "dead_volume_ul",
+                     "safe_min_volume_ul"])):
 
     """
     The ContainerType class holds the capabilities and properties of a
@@ -74,28 +76,35 @@ class ContainerType(namedtuple("ContainerType",
             5
             >>> my_plate.robotize(my_plate.well(3))
             3
+            >>> my_plate.robotize(["A1", "A2"])
+            [0, 1]
 
         Parameters
         ----------
-        well_ref : str, int, Well
-          Well reference to be robotized in string, integer or Well object
-          form.
+        well_ref : str, int, Well, list[str or int or Well]
+            Well reference to be robotized in string, integer or Well object
+            form. Also accepts lists of str, int or Well.
 
         Returns
         ----------
-        well_ref : int
-          Well reference passed as rowwise integer (left-to-right,
-          top-to-bottom, starting at 0 = A1).
+        well_ref : int, list
+            Single or list of Well references passed as rowwise integer
+            (left-to-right, top-to-bottom, starting at 0 = A1).
 
         Raises
         ------
+        TypeError
+            If well reference given is not an accepted type.
         ValueError
             If well reference given exceeds container dimensions.
 
         """
+        if isinstance(well_ref, list):
+            return [self.robotize(well) for well in well_ref]
+
         if not isinstance(well_ref, (basestring, int, Well)):
-            raise TypeError("ContainerType.robotize(): Well reference given "
-                            "is not of type 'str', 'int', or 'Well'.")
+            raise TypeError("ContainerType.robotize(): Well reference (%s) given "
+                            "is not of type 'str', 'int', or 'Well'." % well_ref)
 
         if isinstance(well_ref, Well):
             well_ref = well_ref.index
@@ -141,11 +150,15 @@ class ContainerType(namedtuple("ContainerType",
             'A1'
             >>> my_plate.humanize(5)
             'B3'
+            >>> my_plate.humanize('0')
+            'A1'
 
         Parameters
         ----------
-        well_ref : int
-          Well reference to be humanized in integer form.
+        well_ref : int, str, list[int or str]
+            Well reference to be humanized in integer or string form.
+            If string is provided, it has to be parseable into an int.
+            Also accepts lists of int or str
 
         Returns
         -------
@@ -154,13 +167,23 @@ class ContainerType(namedtuple("ContainerType",
 
         Raises
         ------
+        TypeError
+            If well reference given is not an accepted type.
         ValueError
             If well reference given exceeds container dimensions.
 
         """
-        if not isinstance(well_ref, int):
+        if isinstance(well_ref, list):
+            return [self.humanize(well) for well in well_ref]
+
+        if not isinstance(well_ref, (int, basestring)):
             raise TypeError("ContainerType.humanize(): Well reference given "
-                            "is not of type 'int'.")
+                            "is not of type 'int' or 'str'.")
+        try:
+            well_ref = int(well_ref)
+        except:
+            raise TypeError("ContainerType.humanize(): Well reference given"
+                            "is not parseable into 'int' format.")
         # Check bounds
         if well_ref >= self.well_count or well_ref < 0:
                 raise ValueError("ContainerType.humanize(): Well reference "
@@ -202,146 +225,171 @@ _CONTAINER_TYPES = {
     "384-flat": ContainerType(name="384-well UV flat-bottom plate",
                               well_count=384,
                               well_depth_mm=None,
-                              well_volume_ul=90.0,
+                              well_volume_ul=Unit(90.0, "microliter"),
                               well_coating=None,
                               sterile=False,
                               is_tube=False,
                               capabilities=[],
                               shortname="384-flat",
                               col_count=24,
-                              dead_volume_ul=5,
-                              safe_min_volume_ul=15),
+                              dead_volume_ul=Unit(5, "microliter"),
+                              safe_min_volume_ul=Unit(15, "microliter")),
     "384-pcr": ContainerType(name="384-well PCR plate",
                              well_count=384,
                              well_depth_mm=None,
-                             well_volume_ul=50.0,
+                             well_volume_ul=Unit(50.0, "microliter"),
                              well_coating=None,
                              sterile=None,
                              is_tube=False,
                              capabilities=[],
                              shortname="384-pcr",
                              col_count=24,
-                             dead_volume_ul=2,
-                             safe_min_volume_ul=3),
+                             dead_volume_ul=Unit(2, "microliter"),
+                             safe_min_volume_ul=Unit(3, "microliter")),
     "384-echo": ContainerType(name="384-well Echo plate",
                               well_count=384,
                               well_depth_mm=None,
-                              well_volume_ul=65.0,
+                              well_volume_ul=Unit(65.0, "microliter"),
                               well_coating=None,
                               sterile=None,
                               is_tube=False,
                               capabilities=[],
                               shortname="384-echo",
                               col_count=24,
-                              dead_volume_ul=15,
-                              safe_min_volume_ul=15),
+                              dead_volume_ul=Unit(15, "microliter"),
+                              safe_min_volume_ul=Unit(15, "microliter")),
     "96-flat": ContainerType(name="96-well flat-bottom plate",
                              well_count=96,
                              well_depth_mm=None,
-                             well_volume_ul=340.0,
+                             well_volume_ul=Unit(340.0, "microliter"),
                              well_coating=None,
                              sterile=False,
                              is_tube=False,
                              capabilities=[],
                              shortname="96-flat",
                              col_count=12,
-                             dead_volume_ul=25,
-                             safe_min_volume_ul=65),
+                             dead_volume_ul=Unit(25, "microliter"),
+                             safe_min_volume_ul=Unit(65, "microliter")),
     "96-flat-uv": ContainerType(name="96-well flat-bottom UV transparent \
                                 plate",
                                 well_count=96,
                                 well_depth_mm=None,
-                                well_volume_ul=340.0,
+                                well_volume_ul=Unit(340.0, "microliter"),
                                 well_coating=None,
                                 sterile=False,
                                 is_tube=False,
                                 capabilities=[],
                                 shortname="96-flat-uv",
                                 col_count=12,
-                                dead_volume_ul=25,
-                                safe_min_volume_ul=65),
+                                dead_volume_ul=Unit(25, "microliter"),
+                                safe_min_volume_ul=Unit(65, "microliter")),
     "96-pcr": ContainerType(name="96-well PCR plate",
                             well_count=96,
                             well_depth_mm=None,
-                            well_volume_ul=160.0,
+                            well_volume_ul=Unit(160.0, "microliter"),
                             well_coating=None,
                             sterile=None,
                             is_tube=False,
                             capabilities=[],
                             shortname="96-pcr",
                             col_count=12,
-                            dead_volume_ul=3,
-                            safe_min_volume_ul=5),
+                            dead_volume_ul=Unit(3, "microliter"),
+                            safe_min_volume_ul=Unit(5, "microliter")),
     "96-deep": ContainerType(name="96-well extended capacity plate",
                              well_count=96,
                              well_depth_mm=None,
-                             well_volume_ul=2000.0,
+                             well_volume_ul=Unit(2000.0, "microliter"),
                              well_coating=None,
                              sterile=False,
                              capabilities=[],
                              shortname="96-deep",
                              is_tube=False,
                              col_count=12,
-                             dead_volume_ul=15,
-                             safe_min_volume_ul=30),
+                             dead_volume_ul=Unit(15, "microliter"),
+                             safe_min_volume_ul=Unit(30, "microliter")),
+    "96-v-kf": ContainerType(name="96-well v-bottom King Fisher plate",
+                             well_count=96,
+                             well_depth_mm=None,
+                             well_volume_ul=Unit(200.0, "microliter"),
+                             well_coating=None,
+                             sterile=False,
+                             capabilities=[],
+                             shortname="96-v-kf",
+                             is_tube=False,
+                             col_count=12,
+                             dead_volume_ul=Unit(20, "microliter"),
+                             safe_min_volume_ul=Unit(20, "microliter")),
+    "96-deep-kf": ContainerType(name="96-well extended capacity \
+                                King Fisher plate",
+                                well_count=96,
+                                well_depth_mm=None,
+                                well_volume_ul=Unit(1000.0, "microliter"),
+                                well_coating=None,
+                                sterile=False,
+                                capabilities=[],
+                                shortname="96-deep-kf",
+                                is_tube=False,
+                                col_count=12,
+                                dead_volume_ul=Unit(50, "microliter"),
+                                safe_min_volume_ul=Unit(50, "microliter")),
     "24-deep": ContainerType(name="24-well extended capacity plate",
                              well_count=24,
                              well_depth_mm=None,
-                             well_volume_ul=10000.0,
+                             well_volume_ul=Unit(10000.0, "microliter"),
                              well_coating=None,
                              sterile=False,
                              capabilities=[],
                              shortname="24-deep",
                              is_tube=False,
                              col_count=6,
-                             dead_volume_ul=15,
-                             safe_min_volume_ul=60),
+                             dead_volume_ul=Unit(15, "microliter"),
+                             safe_min_volume_ul=Unit(60, "microliter")),
     "micro-2.0": ContainerType(name="2mL Microcentrifuge tube",
                                well_count=1,
                                well_depth_mm=None,
-                               well_volume_ul=2000.0,
+                               well_volume_ul=Unit(2000.0, "microliter"),
                                well_coating=None,
                                sterile=False,
                                capabilities=[],
                                shortname="micro-2.0",
                                is_tube=True,
                                col_count=1,
-                               dead_volume_ul=15,
-                               safe_min_volume_ul=40),
+                               dead_volume_ul=Unit(15, "microliter"),
+                               safe_min_volume_ul=Unit(40, "microliter")),
     "micro-1.5": ContainerType(name="1.5mL Microcentrifuge tube",
                                well_count=1,
                                well_depth_mm=None,
-                               well_volume_ul=1500.0,
+                               well_volume_ul=Unit(1500.0, "microliter"),
                                well_coating=None,
                                sterile=False,
                                capabilities=[],
                                shortname="micro-1.5",
                                is_tube=True,
                                col_count=1,
-                               dead_volume_ul=15,
-                               safe_min_volume_ul=20),
+                               dead_volume_ul=Unit(15, "microliter"),
+                               safe_min_volume_ul=Unit(20, "microliter")),
     "6-flat": ContainerType(name="6-well cell culture plate",
                             well_count=6,
                             well_depth_mm=None,
-                            well_volume_ul=1500.0,
+                            well_volume_ul=Unit(5000.0, "microliter"),
                             well_coating=None,
                             sterile=False,
                             capabilities=[],
                             shortname="6-flat",
                             is_tube=False,
                             col_count=3,
-                            dead_volume_ul=15,
-                            safe_min_volume_ul=60),
+                            dead_volume_ul=Unit(400, "microliter"),
+                            safe_min_volume_ul=Unit(600, "microliter")),
     "1-flat": ContainerType(name="1-well flat-bottom plate",
                             well_count=1,
                             well_depth_mm=None,
-                            well_volume_ul=80000.0,
+                            well_volume_ul=Unit(80000.0, "microliter"),
                             well_coating=None,
                             sterile=False,
                             capabilities=[],
                             shortname="1-flat",
                             is_tube=False,
                             col_count=1,
-                            dead_volume_ul=36000,
-                            safe_min_volume_ul=40000),
+                            dead_volume_ul=Unit(36000, "microliter"),
+                            safe_min_volume_ul=Unit(40000, "microliter")),
 }
